/*
 * boss2.c
 *
 * Last modified: <2008/10/18 13:51:04 +0900> By SUMIDA
 */
#include <math.h>
#include <SDL.h>
#include "system.h"
#include "texture.h"
#include "ship.h"
#include "shot.h"
#include "effect.h"
#include "enemy.h"

#define SHIP_ACT_MASK    0x001f
#define SHIP_ACT_MOVE    0x0001
#define SHIP_ACT_3WAY    0x0002
#define SHIP_ACT_MISSILE 0x0004
#define SHIP_ACT_TORPEDO 0x0008
#define SHIP_ACT_RING    0x0010

#define HORN_ALIVE  0x0f00
#define HORN_ALIVE1 0x0100
#define HORN_ALIVE2 0x0200
#define HORN_ALIVE3 0x0400
#define HORN_ALIVE4 0x0800

extern SHIP_MGR enemy_mgr;

static Real limit_x[] = {
	IntToReal(-224), IntToReal(-164), IntToReal(-60), IntToReal(-18),
};

/* NOTE : {X̍s

		SHIP_STAT_APPEAR = o

	SHIP_ACT_MOVE
		SHIP_STAT_MOVE   = ㉺ړA[U[Vbg
		SHIP_STAT_NORMAL = ҋ@
	SHIP_ACT_3WAY
		SHIP_STAT_FIRE   = 3wayx3
	SHIP_ACT_MISSILE
		SHIP_STAT_FIRE   = ~TC
	SHIP_ACT_TORPEDO
		SHIP_STAT_FIRE   = q
	SHIP_ACT_RING
		SHIP_STAT_FIRE   = ֔

	ȌAJԂB
 */

void boss2_init(SHIP_INFO *info) {

	REPORT(format("info=%p, id=%d", info, info->id));
	system_info.status |= SYSTEM_STAT_BOSS2;
	info->flag |= SHIP_FLAG_NODAMAGE|SHIP_FLAG_HEAVY|SHIP_FLAG_NOCLIP;
	info->count = 100;
	info->sprite.x  = IntToReal(275);
	info->sprite.y  = IntToReal(HEIGHT - 75);
	info->sprite.dx = IntToReal(1);
	info->sprite.dy = 0;

	enemy_child_new(&enemy_mgr, SHIP_ID_BOSS2_HORN1, info);
	enemy_child_new(&enemy_mgr, SHIP_ID_BOSS2_HORN2, info);
	enemy_child_new(&enemy_mgr, SHIP_ID_BOSS2_HORN3, info);
	enemy_child_new(&enemy_mgr, SHIP_ID_BOSS2_HORN4, info);
}

void boss2(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship) {

	int level = 0;

	if (!(info->flag & HORN_ALIVE1))      level = 3;
	else if (!(info->flag & HORN_ALIVE2)) level = 2;
	else if (!(info->flag & HORN_ALIVE3)) level = 1;
	REPORT(format("level=%d", level));

	if ((info->flag & SHIP_FLAG_DOWN) && (info->flag & HORN_ALIVE)) {
		EFFECT_INFO *effect = effect_new(EFFECT_ID_WAVE);
		effect->sprite.x = info->sprite.x + IntToReal(264);
		effect->sprite.y = info->sprite.y + IntToReal(96);
		effect->sprite.dx = IntToReal(-1);
	}

	if (info->sprite.x < limit_x[level]) {
		info->sprite.dx = IntToReal(1);
	} else {
		info->sprite.dx = 0;
	}

	switch (info->stat) {
	case SHIP_STAT_APPEAR:
		REPORT("SHIP_STAT_APPEAR");
		info->sprite.y  = IntToReal(HEIGHT - 75);
		if (info->count < 0) {
			info->flag &= ~SHIP_FLAG_USERS|SHIP_ACT_MASK|HORN_ALIVE;
			info->stat = SHIP_STAT_MOVE;
			info->count = 60;
			info->sprite.dx = 0;

			/* Nオ */
			info->flag |= SHIP_ACT_MOVE;
			info->sprite.dy = ToReal(-2);
		}
		break;

	case SHIP_STAT_MOVE:
		REPORT("SHIP_STAT_MOVE");
		if (info->count < 0) {
			/* Ԃ玟̃ANV */
			info->stat = SHIP_STAT_NORMAL;
			info->sprite.dy = 0;
			info->count = 60 * 2;
		}

		if (info->flag & SHIP_ACT_MOVE) {
			if (info->sprite.dy < 0 && info->sprite.y < IntToReal(-220)
				|| info->sprite.dy > 0 && info->sprite.y > IntToReal(132)) {
				/* ʒ[Ŕ] */
				info->sprite.dy = -info->sprite.dy;
			}

		} else {

			SURFACE *surface_info = surface_get(info->sprite.surface_id);
			SURFACE *surface_ship = surface_get(ship->sprite.surface_id);

			Real y0 = info->sprite.y + (IntToReal(surface_info->frame_height)>>1);
			Real y1 = ship->sprite.y + (IntToReal(surface_ship->frame_height)>>1);
			Real dy = y1 - y0;

			if (-8 > dy || dy > 8) {
				/* @ɕύX */
				info->sprite.dy = -info->sprite.dy;
			} else {
				/* ̃ANV */
				info->sprite.dy = 0;
				info->count = 60 * 1;
				info->stat  = SHIP_STAT_NORMAL;
			}
		}

		if (info->count & 0x0f == 0) {
			/* [U[Vbg̔ */
			SHOT_INFO *shot = shot_new(shot_mgr, SHOT_ID_RAY1);

			if (shot != NULL) {
				shot->sprite.x = info->sprite.x + IntToReal(245);
				shot->sprite.y = info->sprite.y + IntToReal(168);
				shot->sprite.dx = IntToReal(3);
			}
		}
		break;

	case SHIP_STAT_NORMAL:
		REPORT("SHIP_STAT_NORMAL");
		if (info->count < 0) {
			info->sprite.dy = 0;
			info->count = 10;

			info->stat = SHIP_STAT_FIRE;

			/* 3way x 3ƃ[U[A~TCAq̑I */
			info->flag &= ~SHIP_ACT_MASK;

			switch (level) {
			case 1:
				info->flag |= SHIP_ACT_MISSILE;
				break;
			case 2:
				info->flag |= SHIP_ACT_TORPEDO;
				break;
			case 3:
				info->flag |= SHIP_ACT_RING;
				break;
			default:
				info->flag |= SHIP_ACT_3WAY;
				break;
			}
		}
		break;

	case SHIP_STAT_FIRE:
		REPORT("SHIP_STAT_FIRE");
		if (info->count < 0) {

			info->count = 10;

			if (info->flag & SHIP_ACT_TORPEDO) {

				if (Random(-1, 16) < system_info.level) {
					/* q̔ */
					SHOT_INFO *shot = shot_new(shot_mgr, SHOT_ID_PHOTON);

					if (shot != NULL) {
						shot_fire(shot, info, 0, 0);
						shot->sprite.dy = IntToReal(-8);
						shot->sprite.dx = IntToReal((int)Random(-8, 8));
					}

					info->count += 16 - system_info.level;
				}

			} else if (info->flag & SHIP_ACT_RING) {

				if (Random(-1, 16) < system_info.level) {
					/* [U[ւ̎ˏo */
					SURFACE *surface_info = surface_get(info->sprite.surface_id);
					SURFACE *surface_ship = surface_get(TEX_RING);

					Real x0 = info->sprite.x + (IntToReal(surface_info->frame_width)  >> 1);
					Real y0 = info->sprite.y + (IntToReal(surface_info->frame_height) >> 1);
					Real x1 = IntToReal(surface_ship->frame_width)  >> 1;
					Real y1 = IntToReal(surface_ship->frame_height) >> 1;

					enemy_new(&enemy_mgr, SHIP_ID_RING,
							  x0 - IntToReal(30) - x1,
							  y0 + IntToReal(-30) - y1);

					info->count += 16 - system_info.level;
				}

			} else if (info->flag & SHIP_ACT_MISSILE) {

				/*if (Random(-1, 16) < system_info.level)*/ {
					/* ~TC̔ */
					SURFACE *surface_info = surface_get(info->sprite.surface_id);
					SURFACE *surface_ship = surface_get(TEX_ENEMY9);

					Real x0 = info->sprite.x + (IntToReal(surface_info->frame_width) >> 1);
					Real y0 = info->sprite.y + (IntToReal(surface_info->frame_height) >> 1);
					Real x1 = IntToReal(surface_ship->frame_width) >> 1;
					Real y1 = IntToReal(surface_ship->frame_height) >> 1;

					enemy_new(&enemy_mgr, SHIP_ID_ENEMY9,
							  x0 - IntToReal(30) - x1,
							  y0 + IntToReal(15) - y1);

					info->count = 10 + (16 - system_info.level) * 2;
				}

			} else if (Random(-1, 16) > system_info.level) {
				/* 3way x 3e */

				/* @Zo */
				double angle = sprite_get_angle(&info->sprite, &ship->sprite);
				SHOT_INFO *shot1 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot2 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot3 = shot_new(shot_mgr, SHOT_ID_BALLET);

				SHOT_INFO *shot4 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot5 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot6 = shot_new(shot_mgr, SHOT_ID_BALLET);

				SHOT_INFO *shot7 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot8 = shot_new(shot_mgr, SHOT_ID_BALLET);
				SHOT_INFO *shot9 = shot_new(shot_mgr, SHOT_ID_BALLET);

				if (shot1 != NULL) {
					shot_fire(shot1, info, 0, 0);
					sprite_set_angle(&shot1->sprite, angle, IntToReal(2));
				}
				if (shot2 != NULL) {
					shot_fire(shot2, info, 0, 0);
					sprite_set_angle(&shot2->sprite, angle - 15, IntToReal(2));
				}
				if (shot3 != NULL) {
					shot_fire(shot3, info, 0, 0);
					sprite_set_angle(&shot3->sprite, angle + 15, IntToReal(2));
				}

				if (shot4 != NULL) {
					shot_fire(shot4, info, IntToReal(-25), 0);
					sprite_set_angle(&shot4->sprite, angle - 30, IntToReal(2));
				}
				if (shot5 != NULL) {
					shot_fire(shot5, info, IntToReal(-25), 0);
					sprite_set_angle(&shot5->sprite, angle - 45, IntToReal(2));
				}
				if (shot6 != NULL) {
					shot_fire(shot6, info, IntToReal(-25), 0);
					sprite_set_angle(&shot6->sprite, angle - 60, IntToReal(2));
				}

				if (shot7 != NULL) {
					shot_fire(shot7, info, IntToReal(25), 0);
					sprite_set_angle(&shot7->sprite, angle + 30, IntToReal(2));
				}
				if (shot8 != NULL) {
					shot_fire(shot8, info, IntToReal(25), 0);
					sprite_set_angle(&shot8->sprite, angle + 45, IntToReal(2));
				}
				if (shot9 != NULL) {
					shot_fire(shot9, info, IntToReal(25), 0);
					sprite_set_angle(&shot9->sprite, angle + 60, IntToReal(2));
				}
			}

			if (Random(-4, 18) > system_info.level) {
				info->stat = SHIP_STAT_MOVE;
				info->flag |= SHIP_ACT_MOVE;
				info->count = 60;
				info->sprite.dx = 0;

				if (Random(-2, 2) < 0) {
					info->sprite.dy = IntToReal(-2);
				} else {
					info->sprite.dy = IntToReal(2);
				}
			}
		}
		break;

	default:
		REPORT("illegal action!");
		break;
	}
}

void boss2_term(SHIP_INFO *info) {

	REPORT(format("info=%p", info));
	system_info.status &= ~SYSTEM_STAT_BOSS2;
	enemy_explosion();
}

