/*
 * ring.c
 *
 * Last modified: <2008/10/18 12:11:44 +0900> By SUMIDA
 */
#include <SDL.h>
#include "system.h"
#include "texture.h"
#include "ship.h"
#include "shot.h"
#include "enemy.h"
#include "effect.h"

void ring_init(SHIP_INFO *info) {

	REPORT(format("info=%p, id=%d", info, info->id));
	info->flag |= SHIP_FLAG_HEAVY;
	info->count = 30;
	info->sprite.options &= ~SPRITE_USE_COLLISION;
	info->sprite.dx = IntToReal(1);
	info->sprite.dy = IntToReal((int)Random(-20, -20));
	info->sprite.frame_start    = 0;
	info->sprite.frame_end      = 15;
	info->sprite.frame_interval = 0;
}

void ring(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship) {

	switch (info->stat) {
	case SHIP_STAT_APPEAR:
		info->sprite.dy >>= 1;

		if (info->count < 0) {
			info->count = 5;
			info->stat = SHIP_STAT_MOVE;
			info->sprite.options |= SPRITE_USE_COLLISION;
		}
		break;

	case SHIP_STAT_MOVE:
		info->sprite.dx = IntToReal(1);

		/* ʒ[ŌŒ */
		if (info->sprite.dy < 0 && info->sprite.y < IntToReal(8)
			|| info->sprite.dy > 0 && info->sprite.y > IntToReal(HEIGHT - 64)) {
			//info->sprite.dy = -(info->sprite.dy >> 1);
			info->sprite.dy = 0;
		}

		if (ship->stat != SHIP_STAT_LOST) {
			Real x0 = ((ship->sprite.collision.x0 + ship->sprite.collision.x1) >> 1)
				+ ship->sprite.x - info->sprite.x;
			Real x1 = IntToReal(surface_get(info->sprite.surface_id)->frame_width);

			//printf("x0 = %d, x1 = %d\n", x0, x1);

			if (0 <= x0 && x0 < x1) {
				info->count = 5;
				info->sprite.dx = 0;
				info->stat = SHIP_STAT_NORMAL;
			}
		}
		break;

	case SHIP_STAT_NORMAL:
		if (info->count < 0) {
			info->count = (system_info.level + 1) << 4;
			info->stat = SHIP_STAT_FIRE;
		}
		break;

	case SHIP_STAT_FIRE:
		if (info->count % 7 == 0) {
			/* [U[̔ */
			SHOT_INFO *shot = shot_new(shot_mgr, SHOT_ID_RAY2);

			if (shot != NULL) {
				Real dy = (system_info.level >> 2) + 1;

				if (((ship->sprite.collision.y0 + ship->sprite.collision.y1) >> 1)
					+ ship->sprite.y < info->sprite.y) {
					dy = -dy;
				}

				shot_fire(shot, info, 0, 0);
				shot->sprite.dy = IntToReal(dy);
			}
		}

		if (info->count < 0) {
			info->count = 5;
			info->stat = SHIP_STAT_MOVE;
		}
		break;

	default:
		return;
	}
}

void ring_term(SHIP_INFO *info) {
	REPORT(format("info=%p", info));
}

