/*
 * ship.c
 *
 * Last modified: <2008/10/18 12:12:39 +0900> By SUMIDA
 */
#include <stdlib.h>
#include <string.h>
#include <SDL.h>
#include "system.h"
#include "texture.h"
#include "ship.h"
#include "effect.h"
#include "sound.h"
#include "shot.h"

#define SPRITE_OPT_SHIP  SPRITE_VISIBLE|SPRITE_USE_MOVE|SPRITE_USE_COLLISION

typedef struct SHIP_TYPE {
	int tex_id;
	int strength;
	int point;
	Uint32 options;
	RealRect collision;
} SHIP_TYPE;

SHIP_TYPE ship_type[] = {
	{TEX_ANGEL ,     1,    0,  1|SPRITE_OPT_SHIP , {IntToReal(32), IntToReal(16), IntToReal(48)    , IntToReal(32)   }},
	{TEX_ENEMY1,     1,   10, 10|SPRITE_OPT_SHIP , {IntToReal(2) , IntToReal(2) , IntToReal(60-3)  , IntToReal(60-3) }},
	{TEX_ENEMY2,     2,   20, 11|SPRITE_OPT_SHIP , {IntToReal(10), IntToReal(30), IntToReal(34)    , IntToReal(68)   }},
	{TEX_ENEMY3,    10,   50, 12|SPRITE_OPT_SHIP , {IntToReal(2) , IntToReal(2) , IntToReal(60-3)  , IntToReal(60-3) }},
	{TEX_ENEMY4,    20,  100, 13|SPRITE_OPT_SHIP , {IntToReal(24), IntToReal(40), IntToReal(90)    , IntToReal(66)   }},
	{TEX_ENEMY5,    30,  200, 14|SPRITE_OPT_SHIP , {IntToReal(2) , IntToReal(2) , IntToReal(60-3)  , IntToReal(60-3) }},
	{TEX_ENEMY6,    35,  300, 15|SPRITE_OPT_SHIP , {IntToReal(2) , IntToReal(2) , IntToReal(82)    , IntToReal(112)  }},
	{TEX_ENEMY7,    40,  400, 16|SPRITE_OPT_SHIP , {IntToReal(2) , IntToReal(2) , IntToReal(84)    , IntToReal(112-3)}},
	{TEX_ENEMY8,    50,  500, 17|SPRITE_OPT_SHIP , {IntToReal(32), IntToReal(2) , IntToReal(108)   , IntToReal(124-3)}},
	{TEX_ENEMY9,    25,  150, 18|SPRITE_OPT_SHIP , {IntToReal(36), IntToReal(10), IntToReal(68)    , IntToReal(50)   }},
	{TEX_AXE   ,     1,   10, 19|SPRITE_OPT_SHIP , {IntToReal(4) , IntToReal(10), IntToReal(16-4)  , IntToReal(54-1) }},
	{TEX_PARACHUTE,  1,   10, 20|SPRITE_OPT_SHIP , {IntToReal(4) , IntToReal(10), IntToReal(16-4)  , IntToReal(54-1) }},
	{TEX_BOX   ,    25,  100, 21|SPRITE_OPT_SHIP , {IntToReal(6) , IntToReal(6) , IntToReal(96)    , IntToReal(96)   }},
	{TEX_SNOWFLAKE,  5,   40,  2|SPRITE_OPT_SHIP|SPRITE_FRAME32, {IntToReal(2) , IntToReal(2) , IntToReal(60-3)  , IntToReal(60-3)}},
	{TEX_FIRE  ,    10,   50,  3|SPRITE_OPT_SHIP|SPRITE_FRAME48|SPRITE_LOOP_ANIME|SPRITE_USE_ANIME, {IntToReal(2) , IntToReal(2) , IntToReal(60-3)  , IntToReal(60-3)}},
	{TEX_RING   ,  100,  100, 22|SPRITE_OPT_SHIP|SPRITE_FRAME|SPRITE_LOOP_ANIME|SPRITE_USE_ANIME, {IntToReal(16) , IntToReal(16), IntToReal(96-16)  , IntToReal(64-20) }},
	{TEX_BOSS2_HORN1,  200,  100, 23|SPRITE_OPT_SHIP , {IntToReal(0) , IntToReal(0), IntToReal(86)  , IntToReal(47) }},
	{TEX_BOSS2_HORN2,  200,  100, 23|SPRITE_OPT_SHIP , {IntToReal(0) , IntToReal(0), IntToReal(48)  , IntToReal(92) }},
	{TEX_BOSS2_HORN3,  200,  100, 23|SPRITE_OPT_SHIP , {IntToReal(0) , IntToReal(0), IntToReal(70)  , IntToReal(66) }},
	{TEX_BOSS2_HORN4,  200,  100, 23|SPRITE_OPT_SHIP , {IntToReal(0) , IntToReal(0), IntToReal(58)  , IntToReal(35) }},
	{TEX_BOSS1 ,   500,10000, 24|SPRITE_OPT_SHIP , {IntToReal(156), IntToReal(37), IntToReal(192)   , IntToReal(84)  }},
	{TEX_BOSS2 ,   700,50000, 25|SPRITE_OPT_SHIP , {IntToReal(192), IntToReal(132), IntToReal(286) , IntToReal(204)  }},
#ifdef FOR_1KSP
	{TEX_RUBY  ,     1,  100,  0|SPRITE_OPT_SHIP , {IntToReal(0) , IntToReal(0) , IntToReal(48)    , IntToReal(46)   }},
	{TEX_RAILS ,     1,  100,  0|SPRITE_OPT_SHIP , {IntToReal(0) , IntToReal(0) , IntToReal(48)    , IntToReal(61)   }},
	{TEX_JAVA  ,     1,  100,  0|SPRITE_OPT_SHIP , {IntToReal(0) , IntToReal(0) , IntToReal(33)    , IntToReal(43)   }},
	{TEX_PYTHON,     1,  100,  0|SPRITE_OPT_SHIP , {IntToReal(0) , IntToReal(0) , IntToReal(39)    , IntToReal(39)   }},
#else
	{TEX_ITEM_POWERUP,     1,  100,  0|SPRITE_OPT_SHIP|SPRITE_FRAME32|SPRITE_LOOP_ANIME|SPRITE_USE_ANIME, {IntToReal(0) , IntToReal(0) , IntToReal(32-1)    , IntToReal(32-1)   }},
	{TEX_ITEM_SPEEDUP,     1,  100,  0|SPRITE_OPT_SHIP|SPRITE_FRAME32|SPRITE_LOOP_ANIME|SPRITE_USE_ANIME, {IntToReal(0) , IntToReal(0) , IntToReal(32-1)    , IntToReal(32-1)   }},
	{TEX_ITEM_MISSILE,     1,  100,  0|SPRITE_OPT_SHIP|SPRITE_FRAME32|SPRITE_LOOP_ANIME|SPRITE_USE_ANIME, {IntToReal(0) , IntToReal(0) , IntToReal(32-1)    , IntToReal(32-1)   }},
	{TEX_ITEM_3WAY   ,     1,  100,  0|SPRITE_OPT_SHIP|SPRITE_FRAME32|SPRITE_LOOP_ANIME|SPRITE_USE_ANIME, {IntToReal(0) , IntToReal(0) , IntToReal(32-1)    , IntToReal(32-1)   }},
#endif
	{TEX_STAR  ,    50,  100,  0|SPRITE_OPT_SHIP , {IntToReal(0) , IntToReal(0) , IntToReal(32-1)    , IntToReal(32-1)   }},
	{TEX_STAR  ,     1,  100,  0|SPRITE_OPT_SHIP , {IntToReal(0) , IntToReal(0) , IntToReal(32-1)    , IntToReal(32-1)   }},
	{TEX_STAR2 ,     1,   10,  0|SPRITE_OPT_SHIP , {IntToReal(-4) , IntToReal(-4) , IntToReal(8+4-1) , IntToReal(8+4-1)  }},
};

void ship_init(SHIP_MGR *mgr, SHIP_INFO *infos, size_t number) {

	size_t i;

	if (mgr != NULL) {
		memset(mgr, 0, sizeof(SHIP_MGR));
		mgr->pool = infos;
	}

	memset(infos, 0, sizeof(SHIP_INFO) * number);

	for (i = 0; i < number - 1; i++) {
		infos[i].next = &infos[i + 1];
	}
}

SHIP_INFO *ship_new(SHIP_MGR *mgr, SHIP_ID id) {

	SHIP_INFO *info = mgr->pool;

	if (info != NULL) {
		SHIP_TYPE *p = &ship_type[id];

		mgr->pool   = info->next;
		info->next  = mgr->active;
		info->prev  = NULL;
		if (info->next != NULL) info->next->prev = info;
		mgr->active = info;

		info->id       = id;
		info->stat     = SHIP_STAT_NORMAL;
		info->flag     = 0;
		info->count    = 0;
		info->point    = p->point;
		info->strength = p->strength;
		info->sprite.surface_id = p->tex_id;
		info->sprite.options = p->options;
		info->sprite.frame_start    = 0;
		info->sprite.frame_end      = 1;
		info->sprite.frame_interval = 5;
		info->sprite.collision.x0   = p->collision.x0;
		info->sprite.collision.y0   = p->collision.y0;
		info->sprite.collision.x1   = p->collision.x1;
		info->sprite.collision.y1   = p->collision.y1;
		info->parent   = NULL;
	}

	return info;
}

SHIP_INFO *ship_child_new(SHIP_MGR *mgr, SHIP_ID id, SHIP_INFO *parent) {

	SHIP_INFO *info = mgr->pool;

	if (info != NULL) {
		SHIP_TYPE *p = &ship_type[id];

		mgr->pool   = info->next;
		info->next  = parent->next;
		info->prev  = parent;
		if (info->next != NULL) info->next->prev = info;
		parent->next = info;

		info->id       = id;
		info->stat     = SHIP_STAT_NORMAL;
		info->flag     = 0;
		info->count    = 0;
		info->point    = p->point;
		info->strength = p->strength;
		info->sprite.surface_id = p->tex_id;
		info->sprite.options = p->options;
		info->sprite.frame_start    = 0;
		info->sprite.frame_end      = 1;
		info->sprite.frame_interval = 5;
		info->sprite.collision.x0   = p->collision.x0;
		info->sprite.collision.y0   = p->collision.y0;
		info->sprite.collision.x1   = p->collision.x1;
		info->sprite.collision.y1   = p->collision.y1;
		info->parent   = parent;
	}

	return info;
}

void ship_delete(SHIP_MGR *mgr, SHIP_INFO *info) {

	SHIP_INFO *next = info->next;
	SHIP_INFO *prev = info->prev;

	if (prev == NULL) {
		mgr->active = next;
	} else {
		prev->next = next;
	}
	if (next != NULL) {
		next->prev = prev;
	}

	memset(info, 0, sizeof(SHIP_INFO));
	info->next = mgr->pool;
	mgr->pool = info;
}

int ship_down(SHIP_INFO *info, int *strength, Real dx, Real dy) {

	int score = 0;

	if (info->stat != SHIP_STAT_EXPLOSION && info->stat != SHIP_STAT_LOST) {

		int pow = (*strength > SHOT_LIMIT_STRENGTH)? SHOT_LIMIT_STRENGTH: *strength;

		*strength -= pow;
		if (!(info->flag & SHIP_FLAG_NODAMAGE)) {
			info->strength -= pow;
		}

		if (info->strength > 0) {
			info->flag |= SHIP_FLAG_DOWN;
			if (!(info->flag & SHIP_FLAG_HEAVY)) {
				info->sprite.x += dx >> 1;
				info->sprite.y += dy >> 2;
			}
			sound_effect(SE_SHOT1);

		} else {
			EFFECT_INFO *effect;

			REPORT(format("bomb! (info=%p, id=%d, point=%d)", info, info->id, info->point));
			score = info->point;
			info->stat = SHIP_STAT_EXPLOSION;
			info->count = SHIP_DEF_LOST_WAIT;
			info->sprite.options &= ~SPRITE_USE_MOVE;

			switch (info->id) {
			case SHIP_ID_ME:
				effect = effect_new(EFFECT_ID_DISPERSE);
				sound_effect(SE_BOMB0);
				break;

			case SHIP_ID_BOSS1:
			case SHIP_ID_BOSS2:
				effect = effect_new(EFFECT_ID_BOMB);
				sound_effect(SE_BOMB1);
				break;

			default:
				effect = effect_new(EFFECT_ID_EXPLOSION);
				sound_effect(SE_BOMB0);
				break;
			}

			if (effect != NULL) {
				effect_set_pos(effect, &info->sprite, 0, 0);
			}
		}
	}

	return score;
}
