/*
 * surface.c
 *
 * Last modified: <2008/08/01 05:55:05 +0900> By SUMIDA
 */
#include <stdio.h>
#include <string.h>
#include <SDL.h>
#include <SDL_image.h>
#include "surface.h"

static SURFACE surfaces[SURFACES_MAX];

FUNC_SURFACE_DRAW func_surface_draw[] = {
	NULL,
	surface_draw_frame,
	surface_draw_frame8,
	surface_draw_frame16,
	surface_draw_frame32,
	surface_draw_frame48,
	surface_draw_frame64,
};

static Uint32 surface_get_origin_pixel(SDL_Surface *surface) {

	Uint32 pixel = 0;

	memcpy(&pixel, surface->pixels, surface->format->BytesPerPixel);

	return pixel;
}

void surface_init(void) {
	printf("surface_init ... ");
	memset(surfaces, 0, sizeof(surfaces));
	printf("done.\n");
}

SDL_Surface *surface_load(const char *name, int id) {

	SDL_Surface *surface = NULL;
	SDL_Surface *temp = IMG_Load(name);

	printf("surface_load(%s) ... ", name);

	if (temp == NULL) {
		fprintf(stderr, "Couldn't load: %s\n", SDL_GetError());

	} else {
		/* (x,y)=(0,0)̐F𓧉ߐFƂ */
		SDL_SetColorKey(temp, (SDL_SRCCOLORKEY|SDL_RLEACCEL), surface_get_origin_pixel(temp));

		/* ݂̃rfItH[}bgɕϊ */
		surface = SDL_DisplayFormat(temp);
		SDL_FreeSurface(temp);
		if (temp == NULL) {
			fprintf(stderr, "Couldn't convert sprite: %s\n", SDL_GetError());

		} else {
			SURFACE *p = &surfaces[id];

			/* ȑÕT[tFXj */
			surface_trash(id);

			/* ̃T[tFXo^ */
			p->surface      = surface;
			p->frame_width  = surface->w;
			p->frame_height = surface->h;
		}
	}

	printf("done.\n");

	return surface;
}

void surface_trash(int id) {

	SURFACE *p = &surfaces[id];

	if (p->surface != NULL) {
		SDL_FreeSurface(p->surface);
	}

	memset(p, 0, sizeof(SURFACE));
}

void surface_free(void) {

	int id;

	printf("surface_free ... ");
	for (id = 0; id < SURFACES_MAX; id++) {
		surface_trash(id);
	}
	printf("done.\n");
}

void surface_set_frame(int id, int width, int height) {

	SURFACE *p = &surfaces[id];

	p->frame_width  = width;
	p->frame_height = height;
}

void surface_set_alpha(int id, Uint8 alpha) {

	SURFACE *p = &surfaces[id];

	SDL_SetAlpha(p->surface, SDL_SRCALPHA, alpha);
}

void surface_reset_alpha(int id) {

	SURFACE *p = &surfaces[id];

	SDL_SetAlpha(p->surface, 0, SDL_ALPHA_OPAQUE);
}

void surface_draw(SDL_Surface *screen, int id, int x, int y) {

	SURFACE *p = &surfaces[id];
	SDL_Rect rect, area;

	//printf("surface_draw(%d) ... ", id);

	rect.x = 0;
	rect.y = 0;
	rect.w = p->frame_width;
	rect.h = p->frame_height;

	area.x = x;
	area.y = y;
	area.w = 0;
	area.h = 0;

	SDL_BlitSurface(p->surface, &rect, screen, &area);

	//printf("done.\n");
}

void surface_draw_frame(SDL_Surface *screen, int id, int frame, int x, int y) {

	SURFACE *p = &surfaces[id];
	SDL_Rect rect, area;

	int w = p->surface->w / p->frame_width;
	int sx = (frame % w) * p->frame_width;
	int sy = (frame / w) * p->frame_height;

	rect.x = sx;
	rect.y = sy;
	rect.w = p->frame_width;
	rect.h = p->frame_height;

	area.x = x;
	area.y = y;
	area.w = 0;
	area.h = 0;

	SDL_BlitSurface(p->surface, &rect, screen, &area);
}

void surface_draw_frame8(SDL_Surface *screen, int id, int frame, int x, int y) {

	SURFACE *p = &surfaces[id];
	SDL_Rect rect, area;

	int w = p->surface->w >> 3;
	int sx = (frame % w) << 3;
	int sy = (frame / w) << 3;

	rect.x = sx;
	rect.y = sy;
	rect.w = 8;
	rect.h = 8;

	area.x = x;
	area.y = y;
	area.w = 0;
	area.h = 0;

	SDL_BlitSurface(p->surface, &rect, screen, &area);
}

void surface_draw_frame16(SDL_Surface *screen, int id, int frame, int x, int y) {

	SURFACE *p = &surfaces[id];
	SDL_Rect rect, area;

	int w = p->surface->w >> 4;
	int sx = (frame % w) << 4;
	int sy = (frame / w) << 4;

	rect.x = sx;
	rect.y = sy;
	rect.w = 16;
	rect.h = 16;

	area.x = x;
	area.y = y;
	area.w = 0;
	area.h = 0;

	SDL_BlitSurface(p->surface, &rect, screen, &area);
}

void surface_draw_frame32(SDL_Surface *screen, int id, int frame, int x, int y) {

	SURFACE *p = &surfaces[id];
	SDL_Rect rect, area;

	int w = p->surface->w >> 5;
	int sx = (frame % w) << 5;
	int sy = (frame / w) << 5;

	//printf("frame=%d, w=%d, sx=%d, sy=%d\n", frame, w, sx, sy);

	rect.x = sx;
	rect.y = sy;
	rect.w = 32;
	rect.h = 32;

	area.x = x;
	area.y = y;
	area.w = 0;
	area.h = 0;

	SDL_BlitSurface(p->surface, &rect, screen, &area);
}

void surface_draw_frame48(SDL_Surface *screen, int id, int frame, int x, int y) {

	SURFACE *p = &surfaces[id];
	SDL_Rect rect, area;

	int w = p->surface->w / 48;
	int sx = (frame % w) * 48;
	int sy = (frame / w) * 48;

	//printf("frame=%d, w=%d, sx=%d, sy=%d\n", frame, w, sx, sy);

	rect.x = sx;
	rect.y = sy;
	rect.w = 48;
	rect.h = 48;

	area.x = x;
	area.y = y;
	area.w = 0;
	area.h = 0;

	SDL_BlitSurface(p->surface, &rect, screen, &area);
}

void surface_draw_frame64(SDL_Surface *screen, int id, int frame, int x, int y) {

	SURFACE *p = &surfaces[id];
	SDL_Rect rect, area;

	int w = p->surface->w >> 6;
	int sx = (frame % w) << 6;
	int sy = (frame / w) << 6;

	//printf("frame=%d, w=%d, sx=%d, sy=%d\n", frame, w, sx, sy);

	rect.x = sx;
	rect.y = sy;
	rect.w = 64;
	rect.h = 64;

	area.x = x;
	area.y = y;
	area.w = 0;
	area.h = 0;

	SDL_BlitSurface(p->surface, &rect, screen, &area);
}

SURFACE *surface_get(int id) {

	return &surfaces[id];
}

SDL_bool surface_is_exists(int id) {

	SDL_bool result = SDL_FALSE;

	if (id >= 0 && id < SURFACES_MAX
		&& surfaces[id].surface != NULL) result = SDL_TRUE;

	return result;
}

